﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using Windows.ApplicationModel.Resources.Core;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using System.Collections.Specialized;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

// The data model defined by this file serves as a representative example of a strongly-typed
// model that supports notification when members are added, removed, or modified.  The property
// names chosen coincide with data bindings in the standard item templates.
//
// Applications may use this model as a starting point and build on it, or discard it entirely and
// replace it with something appropriate to their needs.

namespace PdfSamples.Data
{
    /// <summary>
    /// Base class for <see cref="SampleDataItem"/> that defines common properties.
    /// </summary>
    [Windows.Foundation.Metadata.WebHostHidden]
    public abstract class SampleDataCommon : PdfSamples.Common.BindableBase
    {
        private static Uri _baseUri = new Uri("ms-appx:///");

        public SampleDataCommon(String uniqueId, String title, String subtitle, String imagePath)
        {
            this._uniqueId = uniqueId;
            this._title = title;
            this._subtitle = subtitle;
            this._imagePath = imagePath;
            this.Name = uniqueId;
        }

        private string _uniqueId = string.Empty;
        public string UniqueId
        {
            get { return this._uniqueId; }
            set { this.SetProperty(ref this._uniqueId, value); }
        }

        private string _title = string.Empty;
        public string Title
        {
            get { return this._title; }
            set { this.SetProperty(ref this._title, value); }
        }

        private string _subtitle = string.Empty;
        public string Subtitle
        {
            get { return this._subtitle; }
            set { this.SetProperty(ref this._subtitle, value); }
        }

        public string Name { get; set; }

        public DataTemplate Icon
        {
            get
            {
                try
                {
                    return (DataTemplate)App.Current.Resources["IconC1" + Name.Replace(" ", "")];
                }
                catch
                {
                    return (DataTemplate)App.Current.Resources["IconC1Gray"];
                }
            }
        }

        private ImageSource _image = null;
        private String _imagePath = null;
        public ImageSource Image
        {
            get
            {
                if (this._image == null && this._imagePath != null)
                {
                    this._image = new BitmapImage(new Uri(SampleDataCommon._baseUri, this._imagePath));
                }
                return this._image;
            }

            set
            {
                this._imagePath = null;
                this.SetProperty(ref this._image, value);
            }
        }

        public void SetImage(String path)
        {
            this._image = null;
            this._imagePath = path;
            this.OnPropertyChanged("Image");
        }


        public override string ToString()
        {
            return this.Title;
        }
    }

    /// <summary>
    /// Generic item data model.
    /// </summary>
    public class SampleDataItem : SampleDataCommon
    {
        public SampleDataItem(String uniqueId, String title, String subtitle, String imagePath, Type pageType)
            : base(uniqueId, title, subtitle, imagePath)
        {
            _pageType = pageType;
        }

        private Type _pageType;
        public Type PageType
        {
            get { return this._pageType; }
            set { this.SetProperty(ref this._pageType, value); }
        }
    }

    /// <summary>
    /// Creates a collection of groups and items with hard-coded content.
    /// 
    /// SampleDataSource initializes with placeholder data rather than live production
    /// data so that sample data is provided at both design-time and run-time.
    /// </summary>
    public sealed class SampleDataSource
    {
        private static SampleDataSource _sampleDataSource = new SampleDataSource();

        private ObservableCollection<SampleDataItem> _allItems = new ObservableCollection<SampleDataItem>();
        public ObservableCollection<SampleDataItem> AllItems
        {
            get { return this._allItems; }
        }

        public static IEnumerable<SampleDataItem> GetItems(string uniqueId)
        {
            if (!uniqueId.Equals("AllItems")) throw new ArgumentException("Only 'AllItems' is supported as a collection of items");

            return _sampleDataSource.AllItems;
        }

        public static SampleDataItem GetItem(string uniqueId)
        {
            // Simple linear search is acceptable for small data sets
            var matches = _sampleDataSource.AllItems.Where((item) => item.UniqueId.Equals(uniqueId));
            if (matches.Count() == 1) return matches.First();
            return null;
        }

        public SampleDataSource()
        {

            _allItems.Add(new SampleDataItem("Basic Text",
                  "Basic Text",
                  "Shows how to position and align text.",
                  "Assets/BasicText.png",
                  typeof(PdfSamples.Samples.BasicTextPage)));
            _allItems.Add(new SampleDataItem("Images",
                  "Align and Scale Images",
                  "Creates a document with some images. Shows how to align and scale images.",
                  "Assets/MediumGray.png",
                  typeof(PdfSamples.Samples.ImagesPage)));
            _allItems.Add(new SampleDataItem("Graphics",
                   "Graphics",
                   "Shows how to use graphics primitives to draw rectangles, lines, ellipses, etc.",
                   "Assets/Graphics.png",
                   typeof(PdfSamples.Samples.GraphicsPage)));
            _allItems.Add(new SampleDataItem("Quotes",
                   "Quotes",
                   "Creates a document with quotations from www.quoteland.com. Shows DrawString, MeasureString, NewPage.",
                   "Assets/Quotes.png",
                   typeof(PdfSamples.Samples.QuotesPage)));
            //_allItems.Add(new SampleDataItem("Tables",
            //       "Tables",
            //       "Creates a document with NorthWind information. Shows how to do tables with indentation and alignment.",
            //       "Resources/Overview.png",
            //       typeof(PdfSamples.Samples.TOCPage)));
            _allItems.Add(new SampleDataItem("Table of Contents",
                   "Table of Contents",
                   "Creates a document with a Table of Contents. Shows how to move pages within the document.",
                   "Assets/TOC.png",
                   typeof(PdfSamples.Samples.TOCPage)));
            _allItems.Add(new SampleDataItem("Text Flow",
                   "Text Flow",
                   "Shows how to flow text into columns and pages.",
                   "Assets/TextFlow.png",
                   typeof(PdfSamples.Samples.TextFlowPage)));
            _allItems.Add(new SampleDataItem("Paper Sizes",
                   "Paper Sizes",
                   "Creates a document containing multiple paper sizes and orientations.",
                   "Assets/PaperSizes.png",
                   typeof(PdfSamples.Samples.PaperSizesPage)));
            _allItems.Add(new SampleDataItem("Export UI",
                  "Export UI Elements",
                  "Creates a document from UI elements on the page.",
                  "Assets/MediumGray.png",
                  typeof(PdfSamples.Samples.RenderUIPage)));
            
        }
    }
}
