﻿using System;
using System.Linq;
using System.Net;
using System.Windows;
using System.Collections.Generic;
using System.Xml.Linq;
using System.Reflection;
using System.ComponentModel;
using System.Globalization;
using Windows.UI.Xaml;

namespace ControlExplorer
{
    public class FeatureDescription : INotifyPropertyChanged, ISearchable
    {
        private string _name;
        private string _description;
        private bool _isExpanded;
        private Uri _packageUri;
        private string _assemblyName;
        private FrameworkElement _sample;

        // constructor
        public FeatureDescription(ControlDescription control, XElement f)
        {
            Control = control;
            Description = (f.Element("Description") != null) ? f.Element("Description").Value : string.Empty;
            Name = (f.Attribute("name") != null) ? f.Attribute("name").Value : string.Empty;
            AssemblyName = (f.Attribute("assemblyName") != null) ? PlatformUtils.AdjustPlatformName(f.Attribute("assemblyName").Value, false) : string.Empty;
            PackageUri = (f.Attribute("packageUri") != null) ? new Uri(PlatformUtils.AdjustPlatformName(f.Attribute("packageUri").Value, true), UriKind.RelativeOrAbsolute) : null;
            DemoControlTypeName = (f.Attribute("type") != null) ? f.Attribute("type").Value : string.Empty;
            Source = (f.Attribute("source") != null) ? f.Attribute("source").Value : string.Empty;
            Event = (f.Element("Event") != null ? f.Element("Event").Value : "");
            Link = new Uri((control.Name + "/" + Name).Trim().Replace("?", "").Replace("'", "").Replace("(", "").Replace(")", ""), UriKind.Relative);
            SupportThemes = (f.Attribute("supportThemes") != null ? bool.Parse(f.Attribute("supportThemes").Value) : true);
            if (f.Attribute("canViewCode") != null && f.Attribute("canViewCode").Value.Equals("false"))
                ViewCodeVisibility = Visibility.Collapsed;
            else
                ViewCodeVisibility = Visibility.Visible;
            if (f.Element("SubFeatures") != null)
            {
                SubFeatures = (from sf in f.Element("SubFeatures").Elements("SubFeature") select new FeatureDescription(control, sf)).ToList();
                foreach (FeatureDescription sub in SubFeatures)
                {
                    sub.OwnerFeature = this;
                }
            }
            else
            {
                SubFeatures = new List<FeatureDescription>();
            }
            if (f.Element("Properties") != null)
            {
                var properties = from pair in f.Element("Properties").Elements("Property")
                                 select new PropertyAttribute
                                 {
                                     MemberName = pair.Attribute("name").Value,
                                     DisplayName = (pair.Attribute("caption") != null ? pair.Attribute("caption").Value : pair.Attribute("name").Value),
                                     DefaultValue = (pair.Attribute("value") != null ? pair.Attribute("value").Value : null),
                                     Browsable = (pair.Attribute("display") != null ? bool.Parse(pair.Attribute("display").Value) : true),
                                     MinimumValue = (pair.Attribute("minimumValue") != null ? double.Parse(pair.Attribute("minimumValue").Value, CultureInfo.InvariantCulture) : double.NaN),
                                     MaximumValue = (pair.Attribute("maximumValue") != null ? double.Parse(pair.Attribute("maximumValue").Value, CultureInfo.InvariantCulture) : double.NaN),
                                     Tag = (pair.Attribute("nullable") != null ? bool.Parse(pair.Attribute("nullable").Value) : false),
                                 };
                Properties = new List<PropertyAttribute>(properties);
            }
        }

        public FeatureDescription(string p1, string p2, string p3, string p4, string p5, string ITEM_CONTENT, ControlDescription group1)
        {
            Name = p1;
            Description = p3;
        }

        public DataTemplate Icon
        {
            get
            {
                // update Icon
                switch (Type)
                {
                    case FeatureType.RelatedSamples:
                        return (DataTemplate)App.Current.Resources["IconRelatedSamples"];

                    case FeatureType.Default:
                    case FeatureType.Custom:
                    default:
                        //if (SubFeatures.Count() == 0)
                        //{
                        //    return (DataTemplate)App.Current.Resources["IconFeatures"];
                        //}
                        //else
                        //{
                        //    return (DataTemplate)App.Current.Resources["IconFolder"];
                        //}
                        return null;
                }
            }
        }
        public int ID { get; set; }
        // general information
        public string Name
        {
            get
            {
                if (Type == FeatureType.Custom)
                {
                    return _name;
                }
                else
                {
                    return (string.IsNullOrEmpty(_name) ? "See it in action" : _name);
                }
            }
            set
            {
                _name = value;
            }
        }
        public string Description
        {
            get
            {
                if (string.IsNullOrEmpty(_description))
                {
                    if (OwnerFeature != null)
                    {
                        return OwnerFeature.Description;
                    }
                    if (Control != null)
                    {
                        return Control.Description;
                    }
                }
                return _description;
            }
            set
            {
                _description = value;
            }
        }
        public Uri Link { get; set; }

        public string Source { get; set; }
        public FrameworkElement Sample
        {
            get
            {
                // create sample every time. In other case sample is not shown after navigating from it and returning back
                //     if (_sample == null)
                {
                    #region create sample
                    switch (DemoControlTypeName)
                    {
                       #region BarCode
                        case "BarCodeSamples.DemoBarCode":
                            _sample = new BarCodeSamples.DemoBarCode();
                            break;
                        #endregion

                        #region book
                        case "ExtendedSamples.C1BookDemo":
                            _sample = new ExtendedSamples.Samples.C1BookDemo();
                            break;
                        case "ExtendedSamples.C1BookPageSpan":
                            _sample = new ExtendedSamples.Samples.C1BookPageSpan();
                            break;
                        #endregion
                        #region colorPicker
                        case "ExtendedSamples.C1ColorPickerDemo":
                            _sample = new ExtendedSamples.Samples.C1ColorPickerDemo();
                            break;
                        #endregion
                        #region calendar
                        case "C1Calendar_Demo.Samples.DefaultView":
                            _sample = new C1Calendar_Demo.Samples.DefaultView();
                            break;
                        case "C1Calendar_Demo.Samples.BoldedDayTemplate":
                            _sample = new C1Calendar_Demo.Samples.BoldedDayTemplate();
                            break;
                        case "C1Calendar_Demo.Samples.DaySlotTemplateSelector":
                            _sample = new C1Calendar_Demo.Samples.DaySlotTemplateSelector();
                            break;
                        case "C1Calendar_Demo.Samples.CustomData":
                            _sample = new C1Calendar_Demo.Samples.CustomData();
                            break;
                        #endregion
                        #region chart
                        case "ChartSamples.Gallery":
                            _sample = new ChartSamples.Gallery();
                            break;
                        case "ChartSamples.Labels":
                            _sample = new ChartSamples.Labels();
                            break;
                        case "ChartSamples.Radar":
                            _sample = new ChartSamples.Radar();
                            break;
                        case "ChartSamples.Markers":
                            _sample = new ChartSamples.Markers();
                            break;
                        case "ChartSamples.DateAxis":
                            _sample = new ChartSamples.DateAxis();
                            break;
                        case "ChartSamples.InteractiveChart":
                            _sample = new ChartSamples.InteractiveChart();
                            break;
                        case "ChartSamples.RangeBar":
                            _sample = new ChartSamples.RangeBar();
                            break;
                        case "ChartSamples.Dynamic":
                            _sample = new ChartSamples.Dynamic();
                            break;
                        case "ChartSamples.FinancialChart":
                            _sample = new ChartSamples.FinancialChart();
                            break;
                        case "ChartSamples.PieSpin":
                            _sample = new ChartSamples.PieSpin();
                            break;
                        case "ChartSamples.PieAnimation":
                            _sample = new ChartSamples.PieAnimation();
                            break;
                        case "ChartSamples.LoadAnimation":
                            _sample = new ChartSamples.LoadAnimation();
                            break;
                        case "ChartSamples.Selection":
                            _sample = new ChartSamples.Selection();
                            break;
                        //case "ChartSamples.LogAxes":
                        //    _sample = new ChartSamples.LogAxes();
                        //    break;
                        //case "ChartSamples.Functions":
                        //    _sample = new ChartSamples.Functions();
                        //    break;
                        //case "ChartSamples.Parametric":
                        //    _sample = new ChartSamples.Parametric();
                        //    break;
                        //case "ChartSamples.TrendLines":
                        //    _sample = new ChartSamples.TrendLines();
                        //    break;
                        //case "ChartSamples.MovAverage":
                        //    _sample = new ChartSamples.MovAverage();
                        //    break;
                        #endregion
                        #region scheduler
                        case "ScheduleSamples.Samples.DefaultView":
                            _sample = new ScheduleSamples.Samples.DefaultView();
                            break;
                        case "ScheduleSamples.Samples.BusinessObjectsBinding":
                            _sample = new ScheduleSamples.Samples.BusinessObjectsBinding();
                            break;
                        #endregion

                        #region ssrsViewer
                        case "SsrsViewerSamples.Samples.DefaultView":
                            _sample = new SsrsViewerSamples.Samples.DefaultView();
                            break;
                        case "SsrsViewerSamples.Samples.AppBarDemo":
                            _sample = new SsrsViewerSamples.Samples.AppBarDemo();
                            break;
                        #endregion
                        
                        #region tile
                        case "TileSamples.Samples.C1TileListBoxSample":
                            _sample = new TileSamples.Samples.C1TileListBoxSample();
                            break;
                        case "TileSamples.Samples.GridViewSample":
                            _sample = new TileSamples.Samples.GridViewSample();
                            break;
                        case "TileSamples.Samples.ListBoxSample":
                            _sample = new TileSamples.Samples.ListBoxSample();
                            break;
                        case "TileSamples.Samples.ContentSourceSample":
                            _sample = new TileSamples.Samples.ContentSourceSample();
                            break;
                        case "TileSamples.Samples.TwitterFeed":
                            _sample = new TileSamples.Samples.TwitterFeed();
                            break;
                        #endregion
                        #region gauges
                        case "GaugeSamples.RadialGauge":
                            _sample = new GaugeSamples.RadialGauge();
                            break;
                        case "GaugeSamples.SpeedometerGauge":
                            _sample = new GaugeSamples.SpeedometerGauge();
                            break;
                        case "GaugeSamples.VolumeGauge":
                            _sample = new GaugeSamples.VolumeGauge();
                            break;
                        case "GaugeSamples.LinearGauge":
                            _sample = new GaugeSamples.LinearGauge();
                            break;
                        case "GaugeSamples.RulerGauge":
                            _sample = new GaugeSamples.RulerGauge();
                            break;
                        case "GaugeSamples.Rule":
                            _sample = new GaugeSamples.Rule();
                            break;
                        case "GaugeSamples.Clock":
                            _sample = new GaugeSamples.Clock();
                            break;
                        case "GaugeSamples.Automobile":
                            _sample = new GaugeSamples.Automobile();
                            break;
                        case "GaugeSamples.Thermometer":
                            _sample = new GaugeSamples.Thermometer();
                            break;
                        case "GaugeSamples.Knob":
                            _sample = new GaugeSamples.Knob();
                            break;
                        case "GaugeSamples.RegionKnob":
                            _sample = new GaugeSamples.RegionKnob();
                            break;
                        #endregion
                       
                        #region DropDown
                        case "BasicLibrarySamples.View.DropDownDemo":
                            _sample = new BasicLibrarySamples.View.DropDownDemo();
                            break;
                        case "BasicLibrarySamples.View.AutoCompleteDropDown":
                            _sample = new BasicLibrarySamples.View.AutoCompleteDropDown();
                            break;
                        case "BasicLibrarySamples.View.HierarchicalDropDown":
                            _sample = new BasicLibrarySamples.View.HierarchicalDropDown();
                            break;
                        #endregion

                        #region input editors
                        case "C1.Xaml.DateTimeEditors.C1DatePicker":
                            _sample = new C1.Xaml.DateTimeEditors.C1DatePicker()
                            {
                                VerticalAlignment = VerticalAlignment.Top,
                                HorizontalAlignment = HorizontalAlignment.Left,
                                Width = 200,
                                AllowNull = true,
                                Margin = new Thickness(20, 50, 0, 0),
                            };
                            break;
                        case "C1.Xaml.DateTimeEditors.C1TimeEditor":
                            _sample = new C1.Xaml.DateTimeEditors.C1TimeEditor()
                            {
                                VerticalAlignment = VerticalAlignment.Top,
                                HorizontalAlignment = HorizontalAlignment.Left,
                                AllowNull = true,
                                Value = null,
                                MinWidth = 250,
                                Margin = new Thickness(20, 50, 0, 0),
                            };
                            break;
                        case "C1.Xaml.DateTimeEditors.C1DateTimePicker":
                            _sample = new C1.Xaml.DateTimeEditors.C1DateTimePicker()
                            {
                                VerticalAlignment = VerticalAlignment.Top,
                                HorizontalAlignment = HorizontalAlignment.Left,
                                Width = 400,
                                AllowNull = true,
                                Margin = new Thickness(20, 50, 0, 0),
                            };
                            break;
                        case "C1.Xaml.C1DateSelector":
                            _sample = new C1.Xaml.C1DateSelector()
                            {
                                // don't set horizontal alignment and width, as control will adjust itself according to available space
                                VerticalAlignment = VerticalAlignment.Top,
                                Margin = new Thickness(20, 50, 0, 0),
                            };
                            break;
                        case "C1.Xaml.C1TimeSelector":
                            _sample = new C1.Xaml.C1TimeSelector()
                            {
                                // don't set horizontal alignment and width, as control will adjust itself according to available space
                                VerticalAlignment = VerticalAlignment.Top,
                                Margin = new Thickness(20, 50, 0, 0),
                            };
                            break;
                        case "C1.Xaml.C1DateTimeSelector":
                            _sample = new C1.Xaml.C1DateTimeSelector()
                            {
                                // don't set horizontal alignment and width, as control will adjust itself according to available space
                                VerticalAlignment = VerticalAlignment.Top,
                                Margin = new Thickness(20, 50, 0, 0),
                            };
                            break;
                        case "BasicLibrarySamples.View.NumericBoxSample":
                            _sample = new BasicLibrarySamples.View.NumericBoxSample()
                            {
                                VerticalAlignment = VerticalAlignment.Center,
                                HorizontalAlignment = HorizontalAlignment.Center
                            };
                            break;
                        case "BasicLibrarySamples.View.MaskedTextBoxSample":
                            _sample = new BasicLibrarySamples.View.MaskedTextBoxSample()
                            {
                                VerticalAlignment = VerticalAlignment.Center,
                                HorizontalAlignment = HorizontalAlignment.Center
                            };
                            break;
                        #endregion
                        #region layout panels
                        case "WrapPanelSample":
                            _sample = new BasicLibrarySamples.View.WrapPanelSample();
                            break;
                        case "DockPanelSample":
                            _sample = new BasicLibrarySamples.View.DockPanelSample();
                            break;
                        case "UniformGridSample":
                            _sample = new BasicLibrarySamples.View.UniformGridSample();
                            break;
                        case "RadialPanelSample":
                            _sample = new BasicLibrarySamples.View.RadialPanelSample();
                            break;
                        #endregion
                        #region listbox
                        //case "TileListBoxSample":
                        //    _sample = new BasicLibrarySamples.View.TileListBoxSample();
                        //    break;
                        case "ListBoxSample":
                            _sample = new BasicLibrarySamples.View.ListBoxSample();
                            break;
                        #endregion
                        #region flexgrid
                        case "FlexGridSamples.MediaLibrary":
                            _sample = new FlexGridSamples.MediaLibrary();
                            break;
                        case "FlexGridSamples.Financial":
                            _sample = new FlexGridSamples.Financial();
                            break;
                        case "FlexGridSamples.FlexGridDemo":
                            _sample = new FlexGridSamples.FlexGridDemo();
                            break;
                        case "FlexGridSamples.CollectionView":
                            _sample = new FlexGridSamples.CollectionView();
                            break;
                        case "FlexGridSamples.Unbound":
                            _sample = new FlexGridSamples.Unbound();
                            break;
                        //case "FlexGridSamples.YouTube":
                        //    _sample = new FlexGridSamples.YouTube();
                        //    break;
                        case "FlexGridSamples.Editing":
                            _sample = new FlexGridSamples.Editing();
                            break;
                        case "FlexGridSamples.ExcelStyleMerge":
                            _sample = new FlexGridSamples.ExcelStyleMerge();
                            break;
                        case "FlexGridSamples.Printing":
                            _sample = new FlexGridSamples.Printing();
                            break;
                        #endregion
                        #region pdf
                        case "PdfSamples.BasicTextPage":
                            _sample = new PdfSamples.Samples.BasicTextPage();
                            break;
                        case "PdfSamples.ImagesPage":
                            _sample = new PdfSamples.Samples.ImagesPage();
                            break;
                        case "PdfSamples.QuotesPage":
                            _sample = new PdfSamples.Samples.QuotesPage();
                            break;
                        case "PdfSamples.GraphicsPage":
                            _sample = new PdfSamples.Samples.GraphicsPage();
                            break;
                        case "PdfSamples.TOCPage":
                            _sample = new PdfSamples.Samples.TOCPage();
                            break;
                        case "PdfSamples.TextFlowPage":
                            _sample = new PdfSamples.Samples.TextFlowPage();
                            break;
                        case "PdfSamples.PaperSizesPage":
                            _sample = new PdfSamples.Samples.PaperSizesPage();
                            break;
                        case "PdfSamples.RenderUIPage":
                            _sample = new PdfSamples.Samples.RenderUIPage();
                            break;
                        #endregion
                        #region pdfviewer
                        case "PdfViewerSamples.PdfViewerDemoPage":
                            _sample = new PdfViewerSamples.PdfViewerDemoPage();
                            break;
                        case "PdfViewerSamples.LargeFile":
                            _sample = new PdfViewerSamples.LargeFile();
                            break;
                        case "PdfViewerSamples.FindText":
                            _sample = new PdfViewerSamples.FindText();
                            break;
                        case "PdfViewerSamples.BookmarksDemo":
                            _sample = new PdfViewerSamples.BookmarksDemo();
                            break;
                        case "PdfViewerSamples.Thumbnails":
                            _sample = new PdfViewerSamples.Thumbnails();
                            break;
                        case "PdfViewerSamples.Printing":
                            _sample = new PdfViewerSamples.Printing();
                            break;
                        case "PdfViewerSamples.PdfFlipView":
                            _sample = new PdfViewerSamples.PdfFlipView();
                            break;
                        case "PdfViewerSamples.AcroformDemo":
                            _sample = new PdfViewerSamples.AcroformDemo();
                            break;
                        case "PdfViewerSamples.AnnotationDemo":
                            _sample = new PdfViewerSamples.AnnotationDemo();
                            break;
             //           case "PdfViewerSamples.RenderMode":
             //               _sample = new PdfViewerSamples.RenderMode();
             //               break;
                        #endregion
                        #region treeview
                        case "TreeViewSample":
                            _sample = new BasicLibrarySamples.View.TreeViewSample();
                            break;
                        case "TreeViewSamples.Edit":
                            _sample = new TreeViewSamples.Edit();
                            break;
                        case "TreeViewSamples.CheckBox":
                            _sample = new TreeViewSamples.CheckBox();
                            break;
                        case "TreeViewSamples.DragDrop":
                            _sample = new TreeViewSamples.DragDrop();
                            break;
                        case "TreeViewSamples.HierarchicalTemplate":
                            _sample = new TreeViewSamples.HierarchicalTemplate();
                            break;
                        #endregion
                        #region tabcontrol
                        case "EntranceTab":
                            _sample = new BasicLibrarySamples.View.EntranceTab();
                            break;
                        case "TabControlSample":
                            _sample = new BasicLibrarySamples.View.TabControlSample();
                            break;
                        case "ClearStyleTabControl":
                            _sample = new BasicLibrarySamples.View.ClearStyleTabControl();
                            break;
                        #endregion
                        #region collectionview
                        case "BasicLibrarySamples.View.CollectionViewConditions":
                            _sample = new BasicLibrarySamples.View.CollectionViewConditions();
                            break;
                        case "BasicLibrarySamples.View.BindingToC1CollectionView":
                            _sample = new BasicLibrarySamples.View.BindingToC1CollectionView();
                            break;
                        #endregion
                        #region menu
                        case "MenuSample":
                            _sample = new BasicLibrarySamples.View.MenuSample();
                            break;
                        case "VerticalMenu":
                            _sample = new BasicLibrarySamples.View.VerticalMenu();
                            break;
                        case "ContextMenu":
                            _sample = new BasicLibrarySamples.View.ContextMenu();
                            break;
                        case "RadialMenu":
                            _sample = new BasicLibrarySamples.View.RadialMenu();
                            break;
                        #endregion

                        #region misc
                        case "ChartSamples.AxisRangeSlider":
                            _sample = new ChartSamples.AxisRangeSlider();
                            break;
                        case "BasicLibrarySamples.View.RangeSlider":
                            _sample = new BasicLibrarySamples.View.RangeSlider();
                            break;
                        case "BasicLibrarySamples.View.GridSplitterSample":
                            _sample = new BasicLibrarySamples.View.GridSplitterSample();
                            break;
                        #endregion

                        #region richtextbox
                        case "RichTextBoxSamples.DemoRichTextBox":
                            _sample = new RichTextBoxSamples.DemoRichTextBox();
                            break;
                        case "RichTextBoxSamples.SyntaxHighlight":
                            _sample = new RichTextBoxSamples.SyntaxHighlight();
                            break;
                        case "RichTextBoxSamples.RichTextBoxFormatting":
                            _sample = new RichTextBoxSamples.RichTextBoxFormatting();
                            break;
                        case "RichTextBoxSamples.AsYouTypeSpellCheck":
                            _sample = new RichTextBoxSamples.AsYouTypeSpellCheck();
                            break;
                        case "RichTextBoxSamples.Printing":
                            _sample = new RichTextBoxSamples.Printing();
                            break;
                        case "RichTextBoxSamples.AppBarDemo":
                            _sample = new RichTextBoxSamples.AppBarDemo();
                            break;
                        case "RichTextBoxSamples.DemoRtfFilter":
                            _sample = new RichTextBoxSamples.DemoRtfFilter();
                            break;
                        case "RichTextBoxSamples.MultiLanguageSpellCheck":
                            _sample = new RichTextBoxSamples.MultiLanguageSpellCheck();
                            break;
                        #endregion

                        #region image
                        case "ImageSamplesLib2013.DemoGifImage":
                            _sample = new ImageSamplesLib2013.DemoGifImage();
                            break;
                        case "ImageSamplesLib2013.Crop":
                            _sample = new ImageSamplesLib2013.Crop();
                            break;
                        case "ImageSamplesLib2013.FaceWarp":
                            _sample = new ImageSamplesLib2013.FaceWarp();
                            break;
                        #endregion

                        #region maps
                        case "MapsSamples.DemoMaps":
                            _sample = new MapsSamples.DemoMaps();
                            break;
                        case "MapsSamples.CustomTileSource":
                            _sample = new MapsSamples.CustomTileSource();
                            break;
                        case "MapsSamples.Cities":
                            _sample = new MapsSamples.Cities();
                            break;
                        case "MapsSamples.EarthQuake":
                            _sample = new MapsSamples.EarthQuake();
                            break;
                        case "MapsSamples.Flicker":
                            _sample = new MapsSamples.Flicker();
                            break;
                        case "MapsSamples.Grid":
                            _sample = new MapsSamples.Grid();
                            break;
                        case "MapsSamples.MapChart":
                            _sample = new MapsSamples.MapChart();
                            break;
                        case "MapsSamples.Marks":
                            _sample = new MapsSamples.Marks();
                            break;
                        #endregion

                        #region orgchart
                        case "OrgChartSamples.TemplateSample":
                            _sample = new OrgChartSamplesLib2013.TemplateSample();
                            break;
                        case "OrgChartSamples.HierarchicalSample":
                            _sample = new OrgChartSamplesLib2013.HierarchicalSample();
                            break;
                        case "OrgChartSamples.MainSample":
                            _sample = new OrgChartSamplesLib2013.MainSample();
                            break;
                        case "OrgChartSamples.CollapseExpand":
                            _sample = new OrgChartSamplesLib2013.CollapseExpand();
                            break;
                        #endregion
                        #region sparkline
                        case "SparklineSamplesLib2013.MainSample":
                            _sample = new SparklineSamplesLib2013.MainSample();
                            break;
                        case "SparklineSamplesLib2013.AppearanceSample":
                            _sample = new SparklineSamplesLib2013.AppearanceSample();
                            break;
                        case "SparklineSamplesLib2013.RegionSales":
                            _sample = new SparklineSamplesLib2013.RegionSales();
                            break;
                        #endregion
                        #region tileview
                        case "TileViewVirtualization.MainPage":
                            _sample = new TileViewSamples.BlankPage();
                            break;
                        case "TileDashboard.MainPage":
                            _sample = new TileViewSamples.MainPage();
                            break;
                        case "TileDashboard.MainPage2":
                            _sample = new TileViewSamples.MainPage2();
                            break;
                        #endregion
                        #region Zip
                        case "ZipSamples.DemoZip":
                            _sample = new ZipSamples.DemoZip();
                            break;
                        #endregion
                        default:
                            break;
                    }

                    #endregion
                }
                return _sample;
            }
            //set
            //{
            //    _sample = value;
            //    if (Properties != null)
            //    {
            //        foreach (var p in Properties)
            //        {
            //            if (p.MemberName == "VerticalAlignment")
            //            {
            //                _sample.VerticalAlignment = (VerticalAlignment)Enum.Parse(typeof(VerticalAlignment), p.DefaultValue as string, false);
            //            }
            //            else if (p.MemberName == "HorizontalAlignment")
            //            {
            //                _sample.HorizontalAlignment = (HorizontalAlignment)Enum.Parse(typeof(HorizontalAlignment), p.DefaultValue as string, false);
            //            }
            //        }
            //    }
            //}
        }

        public FeatureType Type
        {
            get
            {
                //if (Controls.Count > 0)
                //{
                //    return FeatureType.RelatedSamples;
                //}
                //else
                {
                    return FeatureType.Default;
                }
            }
        }

        public bool SupportThemes { get; private set; }
        public Visibility ThemeVisibility 
        {
            get
            {
                return SupportThemes ? Visibility.Visible : Visibility.Collapsed;
            }
        }

        public bool LocalhostOnly { get; set; }

        public bool IsExpanded
        {
            get
            {
                return _isExpanded;
            }
            set
            {
                _isExpanded = value;
                RaisePropertyChanged("IsExpanded");
            }
        }

        // visual behavior
        public bool ShowAnimation { get; set; }

        // demo
        public string DemoControlTypeName { get; set; }
        public ControlDescription Control { get; set; }
        public Visibility ViewCodeVisibility { get; set; }

        public string SolutionPath;
        public string XamlPath;

        public string FullDemoControlTypeName
        {
            get
            {
                if (string.IsNullOrEmpty(DemoControlTypeName))
                {
                    return PlatformUtils.StripPlatformSuffix(Control.AssemblyName, false) + ".C1" + Control.Name; ;
                }
                else
                {
                    return DemoControlTypeName;
                }
            }
        }

        #region assembly

        public Uri PackageUri
        {
            get
            {
                if (_packageUri != null)
                {
                    return _packageUri;
                }
                if (string.IsNullOrEmpty(DemoControlTypeName))
                {
                    return new Uri(Control.AssemblyName + ".zip", UriKind.RelativeOrAbsolute);
                }
                else
                {
                    return new Uri(AssemblyName + ".xap", UriKind.RelativeOrAbsolute);
                }
            }
            set
            {
                _packageUri = value;
            }
        }

        public string AssemblyName
        {
            get
            {
                //if (!string.IsNullOrEmpty(_assemblyName))
                //{
                //    return _assemblyName;
                //}
                //if (string.IsNullOrEmpty(DemoControlTypeName))
                //{
                //    return Control.AssemblyName;
                //}
                //else
                //{
                //    return PlatformUtils.AdjustPlatformName(DemoControlTypeName.Substring(0, DemoControlTypeName.IndexOf('.')), false);
                //}
                return "";
            }
            set
            {
                _assemblyName = value;
            }
        }

        #endregion

        // default type properties
        public List<PropertyAttribute> Properties { get; set; }

        public Visibility PropertiesVisibility
        {
            get
            {
                return Properties != null && Properties.Count > 0 ? Visibility.Visible : Visibility.Collapsed;
            }
        }
        public string Event { get; set; }

        public IEnumerable<FeatureDescription> SubFeatures { get; set; }
        public FeatureDescription OwnerFeature { get; set; }

        #region ISearchable Members

        public bool Contains(string word)
        {
#if (false) // old version
            bool inRelatedSamples = false;
            if (Type == FeatureType.RelatedSamples)
            {
                foreach (var s in Samples)
                {
                    inRelatedSamples |= s.Contains(word);
                }
            }

            bool inSubFeature = false;
            foreach (var sf in SubFeatures)
            {
                inSubFeature |= sf.Contains(word);
            }

            return inRelatedSamples
                || inSubFeature
                || Name.ToLower().Contains(word.ToLower())
                || Description.ToLower().Contains(word.ToLower());
#endif
            word = word.ToLower();
            bool inSubFeature = false;
            foreach (var sf in SubFeatures)
            {
                inSubFeature |= sf.Contains(word);
            }

            return inSubFeature || Name.ToLower().Contains(word); // || Description.ToLower().Contains(word);
        }

        public bool ContainsAny(string[] searchKeys)
        {
            return searchKeys.Any(key => Contains(key));
        }

        #endregion

        protected void RaisePropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        public event PropertyChangedEventHandler PropertyChanged;

    }

    public enum FeatureType
    {
        Default,
        Custom,
        RelatedSamples
    }
}
